#set( $symbol_pound = '#' )
#set( $symbol_dollar = '$' )
#set( $symbol_escape = '\' )
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ${package}.service;

import ${package}.model.Group;
import ${package}.model.Role;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import javax.inject.Inject;
import javax.inject.Named;
import javax.persistence.Query;
import org.conventionsframework.exception.BusinessException;
import org.conventionsframework.model.WrappedData;
import org.conventionsframework.service.impl.StatefulHibernateService;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.Restrictions;
import org.primefaces.model.SortOrder;

/**
 *
 * @author rmpestano
 */
@Named("groupService")
public class GroupServiceImpl extends StatefulHibernateService<Group, Long> implements GroupService {

    @Inject  
    private RoleService roleService;
    
    @Inject
    private ResourceBundle resourceBundle;

    @Override
    public WrappedData<Group> configFindPaginated(int first, int pageSize, String sortField, SortOrder sortOrder, Map<String, String> columnFilters, Map<String, Object> externalFilter) {

        DetachedCriteria dc = getDetachedCriteria();

        if (columnFilters != null && !columnFilters.isEmpty()) {
            super.addBasicFilterRestrictions(dc, columnFilters);
            String role = columnFilters.get("roles");
            if (role != null && !"all".equalsIgnoreCase(role)) {
                //create join to fetch group roles 
                dc.createAlias("roles", "roles");
                dc.add(Restrictions.eq("roles.name", role));
            }
        }
        if(externalFilter != null && !externalFilter.isEmpty()){
            super.addBasicFilterRestrictions(dc, externalFilter);
            //not show in group popup groups that user being edited already have
            List<Long> userGroups = (List<Long>) externalFilter.get("userGroups");
            if (userGroups != null && !userGroups.isEmpty()) {
                dc.add(Restrictions.not(Restrictions.in("id", userGroups)));
            }
            
           

            Long userId = (Long) externalFilter.get("currentUser");
            if(userId != null){
                //create join to load groups of current user being edited
                dc.createAlias("users", "users");
                dc.add(Restrictions.eq("users.id", userId));
            }
            
        }

        return super.findPaginated(first, pageSize, sortField, sortOrder, dc);

    }
    
 
     /**
     * this method should be in a service that manage roles but we(purposely) 
     * did not provided one so we do that here in groupService
     * 
     * @param groupId
     * @return roles that are not in current group
     */
    @Override
    public List<Role> findAvailableRoles(Group group) {
        
        
        if(group.getRoles() == null || group.getRoles().isEmpty()){
            return roleService.findAll();//if no roles associated all roles are available
        }
        
        List<Long> roleIds = new ArrayList<Long>();
        for (Role role : group.getRoles()) {
            roleIds.add(role.getId());
        }
 
        Query q = this.getEntityManager().createNamedQuery("Role.findRoleNotInGroup");
        q.setParameter("roleIds", roleIds);
        
        return q.getResultList();
        //Same as criteria below
//        DetachedCriteria dc = roleService.getDetachedCriteria();
//        dc.add(Restrictions.not(Restrictions.in("id", roleIds)));
//        return roleService.findByCriteria(dc);
    }

    @Override
    public void beforeRemove(Group entity) {
        if(entity.getRoles() != null && !entity.getRoles().isEmpty()){
            throw new BusinessException(resourceBundle.getString("group.business01"));
        }
        if(entity.getUsers() != null && !entity.getUsers().isEmpty()){
             throw new BusinessException(resourceBundle.getString("group.business02"));
        }
        super.beforeRemove(entity);
    }
    
    
    
}
