#set( $symbol_pound = '#' )
#set( $symbol_dollar = '$' )
#set( $symbol_escape = '\' )
/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ${package}.service;

import ${package}.model.User;
import java.util.Map;
import javax.ejb.Stateful;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.inject.Inject;
import javax.inject.Named;
import org.conventionsframework.exception.BusinessException;
import org.conventionsframework.model.WrappedData;
import org.conventionsframework.service.impl.StatefulHibernateService;
import org.conventionsframework.util.ResourceBundle;
import org.hibernate.criterion.DetachedCriteria;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Restrictions;
import org.primefaces.model.SortOrder;

/**
 *
 * @author rmpestano
 */
@Named("userService")
@Stateful
public class UserServiceImpl extends StatefulHibernateService<User, Long> implements UserService {

    @Inject
    private ResourceBundle resourceBundle;

    @Override
    @TransactionAttribute(TransactionAttributeType.NEVER)
    public WrappedData<User> configFindPaginated(int first, int pageSize, String sortField, SortOrder sortOrder, Map<String, String> columnFilters, Map<String, Object> externalFilter) {

        DetachedCriteria dc = getDetachedCriteria();

        if (columnFilters != null) {
            super.addBasicFilterRestrictions(dc, columnFilters);

            String group = columnFilters.get("groups");
            if (group != null && !"all".equalsIgnoreCase(group)) {
                dc.createAlias("groups", "groups");
                dc.add(Restrictions.eq("groups.name", group));
            }

        }

        return super.findPaginated(first, pageSize, sortField, sortOrder, dc);
    }

    @Override
    public void beforeRemove(User entity) {
        //override to perform logic before removing an entity
        super.beforeRemove(entity);
    }


    @Override
    public void afterRemove(User entity) {
        //override to perform logic after removing an entity
        super.afterRemove(entity);
    }

    @Override
    public void beforeStore(User entity) {
        //override to perform logic before storing an entity
         if (isExistingUser(entity)) {
            throw new BusinessException(resourceBundle.getString("be.user.existing"));
        }
        super.beforeStore(entity);
    }


    @Override
    public void afterStore(User entity) {
        //override to perform logic after storing an entity
        super.afterStore(entity);
    }

    private boolean isExistingUser(User user) {
        DetachedCriteria dc = getDetachedCriteria();
        //used to ignore user id which we are editing
        if (user.getId() != null) {
            dc.add(Restrictions.ne("id", user.getId()));
        }

        if (user != null && !"".endsWith(user.getName())) {
            dc.add(Restrictions.ilike("name", user.getName(), MatchMode.EXACT));
            return (super.getRowCount(dc) > 0);
        }
        return false;
    }

}
